QueryTerminated class

Event emitted when the query terminates.

This is always the final event in the stream and contains the complete query result. After this event, no more events will be emitted.

Inheritance

Constructors

QueryTerminated({required QueryResult result})

Properties

hashCode int
The hash code for this object.
no setterinherited
result QueryResult
The final result of the query operation.
final
runtimeType Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() String
A string representation of this object.
inherited

Operators

operator ==(Object other) bool
The equality operator.
inherited